@@ -7,17 +7,16 @@
 # first written Nov, 2007
 #
 #     This program is free software; you can redistribute it and/or
-#     modify it under the terms of the GNU General Public License, as
-#     published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version. 
+#     modify it under the terms of the GNU General Public License,
+#     version 3, as published by the Free Software Foundation.
 # 
 #     This program is distributed in the hope that it will be useful,
 #     but without any warranty; without even the implied warranty of
-#     merchantability or fitness for a particular purpose.  See the
-#     GNU General Public License for more details.
+#     merchantability or fitness for a particular purpose.  See the GNU
+#     General Public License, version 3, for more details.
 # 
-#     A copy of the GNU General Public License is available at
-#     http://www.r-project.org/Licenses/
+#     A copy of the GNU General Public License, version 3, is available
+#     at http://www.r-project.org/Licenses/GPL-3
 # 
 # Part of the R/qtl package
 # Contains: addint, print.addint, addqtl, addpair,
@@ -696,6 +695,8 @@ function(cross, chr, pheno.col=1, qtl, covar=NULL, formula,
     # are either of the new QTL in the formula?
     g <- c(grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms),
            grep(paste("^[Qq]", n.qtl+2, "$", sep=""), theterms))
+    g1 <- grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms)
+    g2 <- grep(paste("^[Qq]", n.qtl+2, "$", sep=""), theterms)
     if(length(g) == 0)  { # no; add to formula
       newformula1 <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+1,
                                       " + Q", n.qtl+2, " + Q", n.qtl+1,
@@ -705,6 +706,12 @@ function(cross, chr, pheno.col=1, qtl, covar=NULL, formula,
       scanbothways <- FALSE
     }
     else { # need a version without them
+      # first make sure that *both* terms are in the formula
+      if(length(g1)==0) # add Q1
+        formula <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+1, sep=""))
+      if(length(g2)==0) # add Q2
+        formula <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+2, sep=""))
+
       newformula1 <- formula
       newformula2 <- NULL
       
