@@ -2,22 +2,21 @@
 #
 # plot.scanone.R
 #
-# copyright (c) 2001-8, Karl W Broman
-# last modified Aug, 2008
+# copyright (c) 2001-9, Karl W Broman
+# last modified Jun, 2009
 # first written Feb, 2001
 #
 #     This program is free software; you can redistribute it and/or
-#     modify it under the terms of the GNU General Public License, as
-#     published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version. 
+#     modify it under the terms of the GNU General Public License,
+#     version 3, as published by the Free Software Foundation.
 # 
 #     This program is distributed in the hope that it will be useful,
 #     but without any warranty; without even the implied warranty of
-#     merchantability or fitness for a particular purpose.  See the
-#     GNU General Public License for more details.
+#     merchantability or fitness for a particular purpose.  See the GNU
+#     General Public License, version 3, for more details.
 # 
-#     A copy of the GNU General Public License is available at
-#     http://www.r-project.org/Licenses/
+#     A copy of the GNU General Public License, version 3, is available
+#     at http://www.r-project.org/Licenses/GPL-3
 # 
 # Part of the R/qtl package
 # Contains: plot.scanone, 
@@ -41,6 +40,8 @@ function(x,x2,x3,chr,lodcolumn=1,incl.markers=TRUE,xlim, ylim,
      (!missing(x3) && !any(class(x3) == "scanone")))
     stop("Input should have class \"scanone\".")
 
+  if(!is.factor(x$chr)) x$chr <- factor(x$chr, levels=unique(x$chr))
+
   dots <- list(...)
 
   mtick <- match.arg(mtick)
