@@ -2,21 +2,20 @@
 # stepwiseqtl.R
 #
 # copyright (c) 2007-9, Karl W Broman
-# last modified Jan, 2009
+# last modified Jun, 2009
 # first written Nov, 2007
 #
 #     This program is free software; you can redistribute it and/or
-#     modify it under the terms of the GNU General Public License, as
-#     published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version. 
+#     modify it under the terms of the GNU General Public License,
+#     version 3, as published by the Free Software Foundation.
 # 
 #     This program is distributed in the hope that it will be useful,
 #     but without any warranty; without even the implied warranty of
-#     merchantability or fitness for a particular purpose.  See the
-#     GNU General Public License for more details.
+#     merchantability or fitness for a particular purpose.  See the GNU
+#     General Public License, version 3, for more details.
 # 
-#     A copy of the GNU General Public License is available at
-#     http://www.r-project.org/Licenses/
+#     A copy of the GNU General Public License, version 3, is available
+#     at http://www.r-project.org/Licenses/GPL-3
 # 
 # Part of the R/qtl package
 # Contains: stepwiseqtl, calc.plod, countqtlterms, calc.penalties,
@@ -300,7 +299,7 @@ function(cross, chr, pheno.col=1, qtl, formula, max.qtl=10, covar=NULL,
     }
   } # start at null
   else {
-    cat(" ---Starting at a model with", length(qtl$chr), "QTL\n")
+    if(verbose) cat(" ---Starting at a model with", length(qtl$chr), "QTL\n")
     if(refine.locations) {
       if(verbose) cat(" ---Refining positions\n")
       rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
